'use strict';

var vAPI = Object.create(null);

if ( document instanceof window.HTMLDocument ) {
	vAPI.browser = document.documentElement || document.createElement('div');

	if ( !window.requestAnimationFrame ) {
		window.requestAnimationFrame = function(cb) {
			return window.setTimeout(cb, 25);
		};
	}

	if ( vAPI.browser.style ) {
		vAPI._ = vAPI.browser.style;
		vAPI.browser = {
			wheel: 'onwheel' in vAPI.browser ? 'wheel' : 'mousewheel',
			transform: 'webkitTransform' in vAPI._
				? 'webkitTransform'
				: 'transform',
			transitionCSS: 'webkitTransition' in vAPI._
				? '-webkit-transition'
				: 'transition',
			transitionend: 'webkitTransition' in vAPI._
				? 'webkitTransitionEnd'
				: 'transitionend',
			zoomIn: this.chrome || this.mx || this.safari
				? '-webkit-zoom-in'
				: 'zoom-in'
		};
		delete vAPI._;
	}
}

vAPI.buildNodes = function(host, nodes) {
	if ( !host || !Array.isArray(nodes) ) {
		return null;
	}

	if ( !nodes.length ) {
		return host;
	}

	var doc = host.ownerDocument;
	var fragment = doc.createDocumentFragment();

	for ( var i = 0, l = nodes.length; i < l; ++i ) {
		if ( !nodes[i] ) {
			continue;
		}

		if ( typeof nodes[i] === 'string' ) {
			fragment.appendChild(doc.createTextNode(nodes[i]));
			continue;
		}

		var node = doc.createElement(nodes[i].tag);

		if ( nodes[i].attrs ) {
			for ( var attr in nodes[i].attrs ) {
				// bypass CSP
				if ( attr === 'style' ) {
					node.style.cssText = nodes[i].attrs[attr];
				} else {
					node.setAttribute(attr, nodes[i].attrs[attr]);
				}
			}
		}

		if ( nodes[i].nodes ) {
			this.buildNodes(node, nodes[i].nodes);
		} else if ( nodes[i].text ) {
			node.textContent = nodes[i].text;
		}

		fragment.appendChild(node);
	}

	if ( fragment.childNodes.length ) {
		host.appendChild(fragment);
	}

	return host;
};

vAPI.opera = true;
vAPI.browser.irPixelated = '-o-crisp-edges';

Object.defineProperty(vAPI, 'fullScreenElement', {
	get: function() {
		return document.fullscreenElement;
	}
});

Object.defineProperty(vAPI, 'mediaType', {
	get: function() {
		if ( typeof this._mediaType !== 'undefined' ) {
			return this._mediaType;
		}

		var selector = 'link[rel=stylesheet][href^="opera:style/"]';

		if ( !document.head || !document.head.querySelector(selector) ) {
			return this._mediaType = '';
		}

		var media = document.querySelector(
			'body > img[src=""], body > div > video[src=""]'
		);

		if ( !media ) {
			return this._mediaType = '';
		}

		if ( media.parentNode !== document.body ) {
			// media file
			document.body.replaceChild(
				media,
				document.body.firstElementChild
			);
		} else {
			// image
			window.donotrun = media.error = true;
			media.naturalWidth = 0;

			media.onclick = null;
			window.ondragstart = null;
			window.onkeypress = null;
			window.onmousedown = null;
			window.onmouseup = null;
			window.onmousemove = null;
		}

		return this._mediaType = media.localName;
	},
	set: function(type) {
		this._mediaType = type;
	}
});

vAPI.messaging = {
	listen: function(listener, once) {
		if ( this.listener ) {
			opera.extension.removeEventListener('message', this.listener);
		}

		if ( typeof listener !== 'function' ) {
			this.listener = null;
			return;
		}

		this.listener = function(response) {
			if ( once ) {
				vAPI.messaging.listen(null);
			}

			listener(response.data);
		};

		opera.extension.addEventListener('message', this.listener);
	},

	send: function(message, callback) {
		if ( typeof callback === 'function' ) {
			this.listen(callback, true);
		}

		opera.extension.postMessage(message);
	}
};

if ( window.location.protocol === 'widget:' ) {
	document.head.appendChild(
		document.createElement('script')
	).src = './strings.js';

	vAPI.i18n = function(s) {
		if ( !this.i18nData || !this.i18nData.hasOwnProperty(s) ) {
			return s;
		}

		return this.i18nData[s];
	};

	vAPI.insertHTML = function(n, html) {
		n.innerHTML = html;
	};
}

